#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This module allows to select the virtual midi port to send to,
and then starts a bluetooth server over RFCOMM to get the input from the phone
"""

import sys
from time import sleep
import thread
from optparse import OptionParser

try:
    import bluetooth as bt
except ImportError:
    raise Exception("You need the bluetooth library for python installed (pybluez)")

try:
    from rtmidi import RtMidiOut, RtMidiIn, RtError
except ImportError:
    raise Exception("You need the rtmidi library for python installed (rtmidi)")


#bluetooth uuid
UUID = "94f39d29-7d6d-437d-973b-fba39e49d4ee"


def listMidi(midiIn, midiOut):
    #OUT
    print "Enumerating output midi ports..."
    nbPorts = midiOut.getPortCount()
    print "Number of output midi ports detected: %d"%nbPorts
    if nbPorts>0:
        print "output midi ports available:"
        for i in xrange(nbPorts):
            print "[%d]\t%s"%(i,midiOut.getPortName(i))
    else:
        print "No output midi port available on your system, please install a virtual driver first."
    #IN
    #print "listing input midi ports:"
    #nbPorts = midiIn.getPortCount()
    #print "Number of input midi ports detected: %d"%nbPorts
    #if nbPorts>0:
    #    print "input midi ports available:"
    #    for i in xrange(nbPorts):
    #        print "[%d]\t%s"%(i,midiIn.getPortName(i))
    #else:
    #    print "No input midi port available on your system, please install a virtual driver first."

def setupMidi(midiPortOut):
    """
    Opens midi ports for output,
    sends midi events for x and y axis to allow easy midi learn in host applications
    returns the rtmidi in and out out object
    """
    print "================== MIDI SETUP =================="
    if midiPortOut:
        midiOut.openPort(midiPortOut)
    else:
        midiPortOut = -1
        while not midiPortOut in range(midiOut.getPortCount()):
            try:midiPortOut = int(raw_input("enter the port number you wish to connect the midi output "))
            except:midiPortOut=-1
        midiOut.openPort(midiPortOut)
    print "============== MIDI SETUP FINISHED ============"
    return midiOut#, midiIn


def handleMidiFromPhone(midiOut, portNum):
    """
    phone->touch->bt->computer->midi
    """
    print "\n============= BLUETOOTH SERVER LAUNCH ===========\n"
    server_sock=bt.BluetoothSocket(bt.RFCOMM)
    server_sock.bind(("",bt.PORT_ANY))
    server_sock.listen(portNum)
    
    port = server_sock.getsockname()[1]
    
    bt.advertise_service(server_sock, "Server",
                         service_id = UUID,
                         service_classes = [ UUID, bt.SERIAL_PORT_CLASS ],
                         profiles = [ bt.SERIAL_PORT_PROFILE ])
    
    print "\n[MIDIOUT]Waiting for connection on RFCOMM channel/port %d\n" % port
    client_sock, client_info = server_sock.accept()
    print "[MIDIOUT]Phone connected: ", client_info
    
    try:
        while True:
            data = client_sock.recv(255)
            if len(data) == 0: break
            else:
                #print "[MIDIOUT]received : [%s] from device"%(data)
                #wft with the first char??? always '194' but this is nonsense :(
                for i in range(0, len(data), 4):
                    midiOut.sendMessage(ord(data[i+1]), ord(data[i+2]), ord(data[i+3]))
    except IOError:
        pass
    print "[MIDIOUT]disconnected"
    
    client_sock.close()
    server_sock.close()
    print "[MIDIOUT]all sockets closed"



if __name__=="__main__":
    usage = "usage: %prog [-l] [-o1] [-p1]"
    parser = OptionParser(usage=usage, version="%prog 1.0")
    parser.add_option("-o", "--midi-port-out", dest="midiPortOut", type="int",
                  help="OUTPUT Midi port number to use")
    parser.add_option("-p", "--port", dest="port", default=1, type="int",
                      help="Selects the port number the server will listen to")
    (options, args) = parser.parse_args()
    
    midiOut = RtMidiOut()
    
    listMidi(None, midiOut)
    
    setupMidi(options.midiPortOut)
    thread.start_new_thread(handleMidiFromPhone, (midiOut, options.port))
    raw_input("\nPress enter to stop the server and quit")
    #TODO: terminate the bt server cleanly
    stopThreads=True